# -*- coding: utf-8 -*-
"""
/***************************************************************************
 SilLCP
                                 A QGIS plugin
 Plugin pour la création de LCP à partir des outils de la librairie movecost sur les bases du plugin MovecostToQGIS
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                             -------------------
        begin                : 2024-06-03
        git sha              : $Format:%H$
        copyright            : (C) 2024 by Thomas ANDRE
        email                : thomas.andre.archgeo@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

import os
import shutil

from qgis import *
from qgis.PyQt import *
from qgis.core import *
from qgis.utils import *
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from processing.tools import dataobjects
from distutils.dir_util import copy_tree
from processing.tools.system import mkdir, userFolder
import processing
from .Fonctions.Utilitaires import *
from .Fonctions.Fonctions_QGIS import *


# This loads your .ui file so that PyQt can populate your plugin with the elements from Qt Designer
FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'SilLCP_dialog_base.ui'))


class SilLCPDialog(QtWidgets.QDialog, FORM_CLASS):
    def __init__(self, parent=None):
        """Constructor."""
        super(SilLCPDialog, self).__init__(parent)
        # Set up the user interface from Designer through FORM_CLASS.
        # After self.setupUi() you can access any designer object by doing
        # self.<objectname>, and you can use autoconnect slots - see
        # http://qt-project.org/doc/qt-4.8/designer-using-a-ui-file.html
        # #widgets-and-dialogs-with-auto-connect
        self.setupUi(self)

        # Initialisation de certaines fonctions et outils
        self.test()
        self.Choix_Sites.setFilters(QgsMapLayerProxyModel.PointLayer)
        self.Choix_Polygones.setFilters(QgsMapLayerProxyModel.PolygonLayer)
        self.Choix_Origine.setFilters(QgsMapLayerProxyModel.PointLayer)
        self.Choix_Noeuds.setFilters(QgsMapLayerProxyModel.PointLayer)
        self.Choix_Etoile.setFilters(QgsMapLayerProxyModel.PointLayer)
        self.Choix_Etoile.setAllowEmptyLayer(True)
        self.Choix_Etoile.setCurrentIndex(0) 

        # Protection des boutons pour éviter les erreurs
        self.Choix_Origine.setEnabled(False)
        self.OK_Origine.setEnabled(False)
        self.PProche.setEnabled(False)
        self.PCentroide.setEnabled(False)
        self.PEloigne.setEnabled(False)
        self.OK_Type.setEnabled(False)
        self.complet.setChecked(True)
        self.Choix_Etoile.setEnabled(False)

        self.Choix_Etoile.hide()

        # Lien entre toutes les fonctions et les boutons
        self.OK_Sites.clicked.connect(self.Select_Sites)
        self.Choix_Champ.fieldChanged.connect(self.Select_Champ)
        self.OK_Choix.clicked.connect(self.Select_Origine)
        self.OK_Polygones.clicked.connect(self.Select_Polygones)
        self.OK_Origine.clicked.connect(self.Select_Origine_LCP)
        self.OK_Type.clicked.connect(self.Select_Type)
        self.complet.toggled.connect(self.Switch)
        self.etoile.toggled.connect(self.Switch)
        self.OK_Droites.clicked.connect(self.Crea_droites)


        path=os.path.dirname(os.path.abspath(__file__))
        path=path+'//Couches//T//'
        if os.path.isfile(path+'t1.shp') or os.path.isfile(path+'t2.shp'):
            files = os.listdir(path)
            for f in files:
                os.remove(path + f)
        

    def test(self):
        """
        Fonction : Vérifier que le module Processing R Provider est bien installé pour que les scripts puissent être lus
        """
        profile_home = QgsApplication.qgisSettingsDirPath()
        path= os.path.exists(os.path.join(profile_home,'python', 'plugins','processing_r'))
        if path==False:
            QMessageBox.warning(self, "Attention",
                                     "Vous devez d'abord installer et paramétrer le module Processing R Provider pour utiliser celui-ci",
                                     QMessageBox.Ok)
        else :
            a='//rscripts'
            b="processing//rscripts"
            # Accès au dossier où sont stockés les scripts R dans le module
            folder=os.path.dirname(os.path.abspath(__file__))
            source_profile=folder+a
            # Accès au dossier où ils doivent être copiés pour faire fonctionner le module
            new_folder=os.path.dirname(os.path.abspath(folder))
            new_folder=os.path.dirname(os.path.abspath(new_folder))
            new_folder=os.path.dirname(os.path.abspath(new_folder))
            rs=os.path.join(new_folder,b)

            # Vérification si le dossier existe bien
            if not os.path.exists(rs):
                os.makedirs(rs)
            for filename in os.listdir(source_profile):
                # print(str(filename)) # debugtest
                source_file = os.path.join(source_profile, filename)
                dest_file = os.path.join(rs, filename)
                # print(dest_file) # debugtest

                shutil.copy2(source_file, dest_file)
            # print(os.listdir(rs)) # debugtest

        path=os.path.dirname(os.path.abspath(__file__))
        path=path+'//Couches//T//'
        if os.path.isfile(path+'t1.shp') or os.path.isfile(path+'t2.shp'):
            files = os.listdir(path)
            for f in files:
                os.remove(path + f)  


    """
    Ensemble des scripts appelés lors des analyses
    """    
    def on_movecost_pressed(self):
        self.Access_Script("movecost")
        processing.execAlgorithmDialog('r:movecost')

    def on_movebound_pressed(self):
        self.Access_Script("movebound")
        processing.execAlgorithmDialog('r:movebound')

    def on_movecorr_pressed(self):
        self.Access_Script("movecorr")
        processing.execAlgorithmDialog('r:movecorr')

    def on_movealloc_pressed(self):
        self.Access_Script("movealloc")
        processing.execAlgorithmDialog('r:movealloc')

    def on_movecomp_pressed(self):
        self.Access_Script("movecomp")
        processing.execAlgorithmDialog('r:movecomp')

    def on_movenetw_pressed(self):
        self.Access_Script("movenetw")
        processing.execAlgorithmDialog('r:movenetw')

    def on_moverank_pressed(self):
        self.Access_Script("moverank")
        processing.execAlgorithmDialog('r:moverank')

    def on_LCP_pressed(self):
        self.Access_Script("movecost")
        processing.execAlgorithmDialog('r:movecost')

    def on_Reseau_pressed(self):
        self.Access_Script("movenetw")
        processing.execAlgorithmDialog('r:movenetw')

    def on_Corridors_pressed(self):
        self.Access_Script("movecorr")
        processing.execAlgorithmDialog('r:movecorr')

    def on_Isochrones_pressed(self):
        self.Access_Script("movebound")
        processing.execAlgorithmDialog('r:movebound')


    def Access_Script(self,fonction):
        """
        Entree : fonction (str): Fonction à copier
        Fonction : Réalise une copie des scripts pour que l'utilisateur puisse les utiliser et être certain que les scripts soient à jour
        """
        try:
            a='//rscripts'
            b="processing//rscripts"
            # Accès au dossier où sont stockés les scripts R dans le module
            folder=os.path.dirname(os.path.abspath(__file__))
            # print(folder) # debugtest
            source_profile=folder+a
            # print(source_profile) # debugtest
            # Accès au dossier où ils doivent être copiés pour faire fonctionner le module
            new_folder=os.path.dirname(os.path.abspath(folder))
            new_folder=os.path.dirname(os.path.abspath(new_folder))
            new_folder=os.path.dirname(os.path.abspath(new_folder))
            rs=os.path.join(new_folder,b)
            # print(rs) # debugtest

            # Vérification si le dossier existe bien
            if not os.path.exists(rs):
                os.makedirs(rs)

            # Copie des fichiers au moment de l'utilisation, permettant de les mettre automatiquement à jour en cas de modifications
            for filename in os.listdir(source_profile):
                if str(filename) == "{}.rsx".format(fonction) or str(filename) == "{}.rsx.help".format(fonction) :
                    # print(str(filename)) # debugtest
                    source_file = os.path.join(source_profile, filename)
                    dest_file = os.path.join(rs, filename)
                    # print(dest_file) # debugtest

                    shutil.copy2(source_file, dest_file)
            # print(os.listdir(rs)) # debugtest
        except:
            iface.messageBar().pushMessage("Erreur :", "Impossible d'accéder aux scripts R. Vérifier que l'extension Processing R Provider est correctement activée", level=2)



    def Select_Sites(self):
        """
        Fonction : Sélection de la couche dont il faut extraire l'origine et la/les destination(s) des LCP
        """
        self.Couche_Sites=Utilitaires.Recherche_Layer(self.Choix_Sites)
        # Définition de la source de la liste déroulante des entités de la couche sélectionnée
        self.Choix_Champ.setLayer(self.Couche_Sites)


    def Select_Champ(self):
        """
        Fonction : Sélection d'un champ d'attribut pour la sélection du point d'origine des LCP
        """
        self.Champ=self.Choix_Champ.currentField()
        # Changement de champ de référence pour le choix de l'entité
        self.Choix_Depart.setLayer(self.Couche_Sites)
        self.Choix_Depart.setDisplayExpression(self.Champ)


    def Select_Origine(self):
        """
        Fonction : Extraction du point d'origine des LCP, et création d'une couche de destination(s) à partir des points restants
        """
        self.Champ=self.Choix_Champ.currentField()
        Origine=self.Choix_Depart.feature()
        # print(Origine) # debugtest

        # Récupération du SCR de la couche d'entrée pour utiliser le même dans les couches de sortie
        SCR=self.Couche_Sites.crs().authid().replace("EPSG:","")
        # print(self.Couche_Sites.crs().authid().replace("EPSG:","").replace(" ","")) # debugtest
        # Création des couches qui contiendront le point de départ et le(s) point(s) d'arrivée
        Couche_Origine=Utilitaires.Creation_Couche(self,"Point",SCR,Origine[str(self.Champ)],self.Couche_Sites)
        Couche_Destinations=Utilitaires.Creation_Couche(self,"Point",SCR,"Destinations",self.Couche_Sites)

        # Accès aux données de la couche et ajout de l'entité sélectionnée
        Layer_data = Couche_Origine.dataProvider()
        Layer_data.addFeature(Origine)

        # Sélection de tous les autres points de la couche, puis retrait du point de départ pour obtenir le(s) point(s) d'arrivée
        self.Couche_Sites.selectAll()
        Fonctions_QGIS.Selectbylocation_simple(self,self.Couche_Sites,Couche_Origine,3)
        Features=self.Couche_Sites.selectedFeatures()
        # Accès aux données de la couche et ajout des entités restantes
        Layer_data = Couche_Destinations.dataProvider()
        Layer_data.addFeatures(Features)
        # Retrait de la sélection 
        self.Couche_Sites.removeSelection()

        # Possibilité de trier les points en fonction d'une distance si activée
        # self.t12km=Fonctions_QGIS.Buffer(self,Couche_Origine,12000,False,False)
        # self.t100km=Fonctions_QGIS.Buffer(self,Couche_Origine,100000,False,False)
        # # Division des entités de la couche en fonction de leur distance par rapport à l'origine du LCP
        # self.Division(Couche_Destinations)


    def Select_Polygones(self):
        """
        Fonction : Sélection d'une couche de polygones à simplifier en points
        """
        self.Couche_Polygones=Utilitaires.Recherche_Layer(self.Choix_Polygones)
        self.Choix_Origine.setEnabled(True)
        self.OK_Origine.setEnabled(True)


    def Select_Origine_LCP(self):
        """
        Fonction : Sélection de la couche contenant le point d'origine des LCP
        """
        self.Couche_Origine=Utilitaires.Recherche_Layer(self.Choix_Origine)
        self.PCentroide.setEnabled(True)
        self.PProche.setEnabled(True)
        self.PEloigne.setEnabled(True)
        self.OK_Type.setEnabled(True)
    

    def Select_Type(self):
        """
        Fonction : Création d'une couche de points à partir d'une couche de polygones (centroïdes, points le plus proche ou le plus éloigné)
        """
        # print("start") # debugtest
        # Possibilité de trier les points en fonction d'une distance si activée
        # # Création des tampons correspondant aux domaines proches et à la limite du domaine lointain
        # self.t12km=Fonctions_QGIS.Buffer(self,self.Couche_Origine,12000,False,False)
        # self.t100km=Fonctions_QGIS.Buffer(self,self.Couche_Origine,100000,False,False)
        # Vérification des SCR des deux couches pour s'assurer qu'ils sont les mêmes pour le calcul de distance
        SCR1=self.Couche_Polygones.crs().authid().replace("EPSG:","")
        SCR2=self.Couche_Origine.crs().authid().replace("EPSG:","")

        # Création de la couche de centroïdes si sélectionnée
        if self.PCentroide.isChecked():
            # Utilisation de l'outil centroïde de QGIS
            Centroides=Fonctions_QGIS.Centroide(self,self.Couche_Polygones)
            Centroides=Fonctions_QGIS.Reprojeter(self,Centroides,SCR2)
            Centroides.setName("Centroides")
            QgsProject.instance().addMapLayer(Centroides)

            # Division des entités de la couche en fonction de leur distance par rapport à l'origine du LCP
            # self.Division(Centroides)

        # Ne déclencher ces étapes que si besoin de parcourir l'ensemble des éléments de la couche
        if self.PProche.isChecked() or self.PEloigne.isChecked() :
            # S'ils sont différents, reprojeter en fonction du SCR du point d'origine
            if SCR1!=SCR2 :
                self.Couche_Polygones=Fonctions_QGIS.Reprojeter(self,self.Couche_Polygones,SCR2)
            # Récupération des coordonnées de l'origine des LCP
            for Feat in self.Couche_Origine.getFeatures() :
                xa=Feat.geometry().asPoint()[0]
                ya=Feat.geometry().asPoint()[1]

            # Création d'une couche temporaire qui recevra les différentes entités de la couche à transformer progressivement
            Tempo=Utilitaires.Creation_Couche(self,"Polygon",SCR2,"Tempo",self.Couche_Polygones)
            Layer_data = Tempo.dataProvider()

            # Création des listes des points minimaux et maximaux retenus
            liste_min=[]
            liste_max=[]

            # Parcours de l'ensemble des entités de la couche à transformer
            for Feat in self.Couche_Polygones.getFeatures() :
                # Ajouter l'entité à la couche temporaire
                Layer_data.addFeature(Feat)
                # Transformation du polygone en un ensemble de points 
                Sommets=Fonctions_QGIS.Extraction_Sommets(self,Tempo)
                # Recherche des points le plus proche et le plus éloigné de l'entité
                Feat_min,Feat_max=self.Recup_Sommets(Sommets,xa,ya)
                # Ajout des points identifiés dans la liste finale
                liste_min.append(Feat_min)
                liste_max.append(Feat_max)
                # Suppression de l'entité de la couche temporaire pour la remplacer
                Layer_data.truncate()

            # Retrait de la couche du projet, car inutile
            QgsProject.instance().removeMapLayer(Tempo)
        
            # Création de la couche des points les plus proches si sélectionnée
            if self.PProche.isChecked() :
                # Création de la couche
                Proche=Utilitaires.Creation_Couche(self,"Point",SCR2,"Points_+_Proches",Sommets)
                # Accès aux entités de la couche
                Layer_data = Proche.dataProvider()
                # Ajout des points identifiés à la couche
                Layer_data.addFeatures(liste_min)

                # Possibilité de trier les points en fonction d'une distance si activée
                # # Division des entités de la couche en fonction de leur distance par rapport à l'origine du LCP
                self.Division(Proche)

            if self.PEloigne.isChecked() :
                # Création de la couche
                Eloigne=Utilitaires.Creation_Couche(self,"Point",SCR2,"Points_+_Eloignes",Sommets)
                # Accès aux entités de la couche
                Layer_data = Eloigne.dataProvider()
                # Ajout des points identifiés à la couche
                Layer_data.addFeatures(liste_max)

                # Possibilité de trier les points en fonction d'une distance si activée
                # # Division des entités de la couche en fonction de leur distance par rapport à l'origine du LCP
                self.Division(Eloigne)

        self.Choix_Origine.setEnabled(False)
        self.OK_Origine.setEnabled(False)
        self.PProche.setChecked(False)
        self.PCentroide.setChecked(False)
        self.PEloigne.setChecked(False)
        self.PProche.setEnabled(False)
        self.PCentroide.setEnabled(False)
        self.PEloigne.setEnabled(False)
        self.OK_Type.setEnabled(False)


    def Recup_Sommets(self,Sommets,xa,ya):
        """
        Entrée : Sommets (list): Liste de sommets à trier
                 xa,ya (int ou float): Coordonnées du point de référence
        Fonction : Récupération des sommets les plus proche et éloigné d'un point
        Sortie : Points le plus proche et plus éloigné de l'origine du LCP (VectorLayer)
        """
        # Création de valeur pour initialiser la recherche
        dist_min=0
        dist_max=0

        # Recherche dans l'ensemble des sommets du polygone transformé
        for Sommet in Sommets.getFeatures() :
            # Récupération des coordonnées du sommet
            xb=Sommet.geometry().asPoint()[0]
            yb=Sommet.geometry().asPoint()[1]
            # Calcule de la distance séparant le sommet de l'origine du LCP
            dist=(((xa-xb)**2)+((ya-yb)**2))**0.5
            # Vérification s'il est le plus loin ou le plus proche de l'origine du LCP
            if dist < dist_min or dist_min==0 :
                Feat_min=Sommet
                dist_min=dist
            if dist > dist_max or dist_max==0 :
                Feat_max=Sommet
                dist_max=dist

        return Feat_min,Feat_max


    def Division(self,Couche):
        """
        Entrée : Couche (VectorLayer): Couche à modifier
        Fonction : Ajouter un champ et diviser les entités d'une couche en fonction de leur distance par rapport à l'origine des LCP
        (Non conservée, car spécifique à une étude menée, mais possibilié de le modifier)
        """
        if self.PProche.isChecked() or self.PEloigne.isChecked() :
            # Suppression de colonnes dans la table attributaire créées durant le processus, mais non-utiles 
            count=Couche.fields().count()
            ind_list=list((count-1,count-2,count-3,count-4,count-5,count-6))
            # remove multiple fields with a list of indices
            Couche.dataProvider().deleteAttributes(ind_list)
            Couche.updateFields()

        # # Accès aux données de la couche
        # Layer_data = Couche.dataProvider()
        # # Création d'un nouveau champ
        # Layer_data.addAttributes([QgsField("Classe_Distance", QVariant.String)])
        # Couche.updateFields()
        # # Récupération de l'id du champ pour sa complétion
        # field_idx = Couche.fields().indexOf("Classe_Distance")

        # # Sélection des points dans le domaine proche (moins de 12km)
        # Fonctions_QGIS.Selectbylocation_simple(self,Couche,self.t12km,0)
        # with edit(Couche):
        #     for feature in Couche.selectedFeatures():
        #         # Changement de la valeur du champ
        #         Couche.changeAttributeValue(feature.id(), field_idx, "<12km")
        
        # # Sélection des points dans la zone intermédiaire (entre 12 et 100km)
        # Fonctions_QGIS.Selectbylocation_simple(self,Couche,self.t100km,0)
        # Fonctions_QGIS.Selectbylocation_simple(self,Couche,self.t12km,3)
        # with edit(Couche):
        #     for feature in Couche.selectedFeatures():
        #         # Changement de la valeur du champ
        #         Couche.changeAttributeValue(feature.id(), field_idx, "12km->100km")

        # # Sélection des points dans le domaine lointain
        # Couche.selectAll()
        # Fonctions_QGIS.Selectbylocation_simple(self,Couche,self.t100km,3)
        # with edit(Couche):
        #     for feature in Couche.selectedFeatures():
        #         # Changement de la valeur du champ
        #         Couche.changeAttributeValue(feature.id(), field_idx, ">100km")

        # Suppresion de la sélection
        Couche.removeSelection()

    def Switch(self):
        if self.complet.isChecked() :
            self.label_specifique.setText("Noeuds du réseau :")
            self.label_specifique2.setText("")
            self.complet.setChecked(False)
            self.Choix_Etoile.hide()
            self.Choix_Etoile.setCurrentIndex(0) 
            self.Choix_Etoile.setEnabled(False)
        if self.etoile.isChecked() :
            self.label_specifique.setText("Noeud central : ")
            self.label_specifique2.setText("Noeuds à relier : ")
            self.etoile.setChecked(True)
            self.Choix_Etoile.show()
            self.Choix_Etoile.setEnabled(True)
            self.Choix_Etoile.setCurrentIndex(0) 
            

    def Crea_droites(self):
        """
        Fonction = Création d'un réseau de lignes droites entre plusieurs points, ou entre une ou plusieurs origine(s) et plusieurs destinations
        """
        noeuds=Utilitaires.Recherche_Layer(self.Choix_Noeuds)
        SCR=noeuds.crs().authid().replace("EPSG:","")
        Couches_creee=QgsVectorLayer("{}?crs=epsg:{}".format('LineString',SCR), "Reseau_droites", "memory")
        Layer_data = Couches_creee.dataProvider()

        if self.complet.isChecked():
            Layer_data.addAttributes([QgsField("Id", QVariant.String),QgsField("IdCouple", QVariant.String),QgsField("IdA", QVariant.Int),QgsField("IdB", QVariant.Int),QgsField("Distance", QVariant.Double)])
            Couches_creee.setName("Reseau_droites_complet")

        if self.etoile.isChecked():
            arrivees=Utilitaires.Recherche_Layer(self.Choix_Etoile)
            Layer_data.addAttributes([QgsField("Id", QVariant.String),QgsField("Id_Fin", QVariant.Int),QgsField("Distance", QVariant.Double)])
            Couches_creee.setName("Reseau_droites_etoile")

        Couches_creee.updateFields()

        # Ajout de la couche à la carte
        QgsProject.instance().addMapLayer(Couches_creee)

        if self.complet.isChecked():
            self.Reseau_complet(noeuds,Couches_creee)
                
        if self.etoile.isChecked() :
            self.Reseau_etoile(noeuds,arrivees,Couches_creee)


    def Reseau_complet(self,reseau,Couches_creee):
        """
        Entrée(s) = reseau (VectorLayer) : Couche des noeuds du réseau
                    Couches_creee (VectorLayer) : Couche créée pour accueillir le réseau
        Fonction = Création d'un réseau complet entre l'ensemble des noeuds
        """
        Layer_data = Couches_creee.dataProvider()
        liste=[]
        ref=[]
        for noeud in reseau.getFeatures():
            liste.append(noeud)
            ref.append(noeud)
        m=0
        while len(liste)!=1:
            depart=liste[0]
            id1=ref.index(depart)+1
            for arrivee in liste :
                id2=ref.index(arrivee)+1
                if arrivee != depart :
                    points = []
                    points.append(depart.geometry().asPoint())
                    points.append(arrivee.geometry().asPoint())
                    trace = QgsFeature()
                    trace.setGeometry(QgsGeometry.fromPolylineXY(points))
                    dist=trace.geometry().length()
                    Layer_data.addFeature(trace)

                    if id1<id2 :
                        ids=str(id1)+"-"+str(id2)
                    else :
                        ids=str(id2)+"-"+str(id1)

                    with edit(Couches_creee):
                        for feat in Couches_creee.getFeatures() :
                            champs=feat.fields().names()
                            if feat[champs[0]]==None :
                                Couches_creee.changeAttributeValue(feat.id(), 0, m)
                                Couches_creee.changeAttributeValue(feat.id(), 1, ids)
                                Couches_creee.changeAttributeValue(feat.id(), 2, id1)
                                Couches_creee.changeAttributeValue(feat.id(), 3, id2)
                                Couches_creee.changeAttributeValue(feat.id(), 4, dist/1000)
                    m+=1
            del liste[0]


    def Reseau_etoile(self,reseau,arrivees,Couches_creee):
        """
        Entrée(s) = reseau (VectorLayer) : Couche des noeuds de destination
                    arrivees (VectorLayer) : Couche des noeuds centraux
                    Couches_creee (VectorLayer) : Couche créée pour accueillir le réseau
        Fonction = Création d'un réseau complet entre l'ensemble des noeuds
        """
        Layer_data = Couches_creee.dataProvider()
        listen=[]
        for noeud in arrivees.getFeatures():
            listen.append(noeud)
        liste=[]
        for noeud in reseau.getFeatures():
            liste.append(noeud)
        m=0
        for centre in listen :
            for arrivee in liste :
                points = []
                points.append(centre.geometry().asPoint())
                points.append(arrivee.geometry().asPoint())
                trace = QgsFeature()
                trace.setGeometry(QgsGeometry.fromPolylineXY(points))
                dist=trace.geometry().length()
                if dist != 0 :
                    Layer_data.addFeature(trace)

                    with edit(Couches_creee):
                            for feat in Couches_creee.getFeatures() :
                                champs=feat.fields().names()
                                if feat[champs[0]]==None :
                                    Couches_creee.changeAttributeValue(feat.id(), 0, m)
                                    Couches_creee.changeAttributeValue(feat.id(), 1, centre.id()+1)
                                    Couches_creee.changeAttributeValue(feat.id(), 2, dist/1000)
                    m+=1


